(* Copyright (c) 2011. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: ClientDataSets" by Cary Jensen

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: ClientDataSets"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/cdsbook
*)
unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DB, DBClient, StdCtrls, ExtCtrls, Grids, DBGrids,
  DBCtrls, IniFiles, ShellAPI, Provider, DBTables;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    DataSource1: TDataSource;
    Splitter1: TSplitter;
    MergeCacheButton: TButton;
    ClearCacheButton: TButton;
    UndoLastChangeButton: TButton;
    RevertRecordButton: TButton;
    SetSavePointButton: TButton;
    RevertToSavePointButton: TButton;
    LoadFromFileButton: TButton;
    FilterChangedRecords: TCheckBox;
    ClientDataSet1: TClientDataSet;
    ClientDataSet1Status: TStringField;
    ViewXMLButton: TButton;
    ApplyUpdatesButton: TButton;
    Table1: TTable;
    DataSetProvider1: TDataSetProvider;
    OpenCloseButton: TButton;
    SaveWithParamsButton: TButton;
    Label1: TLabel;
    OptionalDataEdit: TEdit;
    SaveToFileButton: TButton;
    LoadWithOptionalParams: TButton;
    ShowDeltaButton: TButton;
    ClientDataSet1CustNo: TFloatField;
    ClientDataSet1Company: TStringField;
    ClientDataSet1Addr1: TStringField;
    ClientDataSet1Addr2: TStringField;
    ClientDataSet1City: TStringField;
    ClientDataSet1State: TStringField;
    ClientDataSet1Zip: TStringField;
    ClientDataSet1Country: TStringField;
    ClientDataSet1Phone: TStringField;
    ClientDataSet1FAX: TStringField;
    ClientDataSet1TaxRate: TFloatField;
    ClientDataSet1Contact: TStringField;
    ClientDataSet1LastInvoiceDate: TDateTimeField;
    EnableDisableChangeCache: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure ClientDataSet1CalcFields(DataSet: TDataSet);
    procedure MergeCacheButtonClick(Sender: TObject);
    procedure ClearCacheButtonClick(Sender: TObject);
    procedure UndoLastChangeButtonClick(Sender: TObject);
    procedure RevertRecordButtonClick(Sender: TObject);
    procedure SetSavePointButtonClick(Sender: TObject);
    procedure RevertToSavePointButtonClick(Sender: TObject);
    procedure LoadFromFileButtonClick(Sender: TObject);
    procedure FilterChangedRecordsClick(Sender: TObject);
    procedure ViewXMLButtonClick(Sender: TObject);
    procedure OpenCloseButtonClick(Sender: TObject);
    procedure DataSource1DataChange(Sender: TObject; Field: TField);
    procedure ClientDataSet1AfterClose(DataSet: TDataSet);
    procedure ClientDataSet1AfterOpen(DataSet: TDataSet);
    procedure ApplyUpdatesButtonClick(Sender: TObject);
    procedure ShowDeltaButtonClick(Sender: TObject);
    procedure SaveToFileButtonClick(Sender: TObject);
    procedure LoadWithOptionalParamsClick(Sender: TObject);
    procedure SaveWithParamsButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure EnableDisableChangeCacheClick(Sender: TObject);
  private
    DataDir: String;
    XMLFileName: String;
    { Private declarations }
    SavedSavePoint: Integer;
    procedure UpdateButtonStatuses;
    //Ini: TIniFile;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses cacheformu;

{$R *.dfm}

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if ClientDataSet1.State in [dsEdit, dsInsert] then
    ClientDataSet1.Post;
  if ClientDataSet1.ChangeCount > 0 then
  begin
    if MessageDlg('There are changes. Do you want to save them?',
       mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    begin
      //Use 0 so that an exception will be raised if
      //all changes cannot be applied.
      ClientDataSet1.ApplyUpdates(0);
    end;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  CustomerTable: TTable;
begin
  DataDir := ExtractFilePath(Application.ExeName);
  XMLFileName := DataDir + 'cdsdata.xml';
  if not Table1.Exists then
  begin
    CustomerTable := TTable.Create(Self);
    try
      CustomerTable.DatabaseName := 'DBDEMOS';
      CustomerTable.TableName := 'CUSTOMER.DB';
      Table1.Tablename := 'CustCopy.DB';
      Table1.BatchMove(CustomerTable, batCopy);
      Table1.AddIndex('','CustNo',[ixPrimary, ixUnique]);
      Table1.AddIndex('ByCompany','Company',[ixCaseInsensitive]);
      Table1.Open;
    finally
      CustomerTable.Free;
    end;
  end;
end;

procedure TForm1.ClientDataSet1AfterClose(DataSet: TDataSet);
begin
  UpdateButtonStatuses;
  SavedSavePoint := -1;
end;

procedure TForm1.ClientDataSet1AfterOpen(DataSet: TDataSet);
begin
  SaveToFileButton.Enabled := True;
  SaveWithParamsButton.Enabled := True;
end;

procedure TForm1.ClientDataSet1CalcFields(DataSet: TDataSet);
begin
  case ClientDataSet1.UpdateStatus of
    usModified: ClientDataSet1Status.Value := 'Modified';
    usDeleted: ClientDataSet1Status.Value := 'Deleted';
    usInserted: ClientDataSet1Status.Value := 'Inserted';
    usUnModified: ClientDataSet1Status.Value := 'Unmodified';
  end;
end;

procedure TForm1.MergeCacheButtonClick(Sender: TObject);
begin
  ClientDataSet1.MergeChangeLog;
end;

procedure TForm1.OpenCloseButtonClick(Sender: TObject);
begin
  if not ClientDataSet1.Active then
  begin
    ClientDataSet1.Open;
    OpenCloseButton.Caption := 'Close ClientDataSet';
  end
  else
  begin
    ClientDataSet1.Close;
    OpenClosebutton.Caption := 'Open from DataSetProvider';
  end;
end;

procedure TForm1.ApplyUpdatesButtonClick(Sender: TObject);
begin
  ClientDataSet1.ApplyUpdates(-1);
end;

procedure TForm1.LoadWithOptionalParamsClick(Sender: TObject);
var
  Data: OleVariant;
begin
  ClientDataSet1.LoadFromFile(XMLFileName);
  OpenCloseButton.Caption := 'Close ClientDataSet';
  UpdateButtonStatuses;
  Data := ClientDataSet1.GetOptionalParam('MyData');
  if not VarIsClear(Data) then
    OptionalDataEdit.Text := DateTimeToStr(Data)
  else
    ShowMessage('No optional data found');
end;

procedure TForm1.EnableDisableChangeCacheClick(Sender: TObject);
begin
  ClientDataSet1.LogChanges := EnableDisableChangeCache.Checked;
end;

procedure TForm1.ClearCacheButtonClick(Sender: TObject);
begin
  ClientDataSet1.CancelUpdates;
end;

procedure TForm1.UndoLastChangeButtonClick(Sender: TObject);
begin
  ClientDataSet1.UndoLastChange(False);
end;

procedure TForm1.RevertRecordButtonClick(Sender: TObject);
begin
  if ClientDataSet1.UpdateStatus <> usUnModified then
    ClientDataSet1.RevertRecord
  else
    ShowMessage('The current record is not in the change cache');
end;

procedure TForm1.SaveToFileButtonClick(Sender: TObject);
begin
  ClientDataSet1.SaveToFile(XMLFileName);
end;

procedure TForm1.SaveWithParamsButtonClick(Sender: TObject);
begin
  ClientDataSet1.SetOptionalParam('MyData', VarFromDateTime(now));
  ClientDataSet1.SaveToFile(XMLFileName)
end;

procedure TForm1.UpdateButtonStatuses;
var
  Caching: Boolean;
  Loadable: Boolean;
  Active: Boolean;
begin
  Caching := ClientDAtaSet1.ChangeCount > 0;
  Loadable := FileExists(XMLFileName);
  Active := ClientDataSet1.Active;
  ApplyUpdatesButton.Enabled := Caching;
  MergeCacheButton.Enabled := Caching;
  ClearCacheButton.Enabled := Caching;
  UndoLastChangeButton.Enabled := Caching;
  RevertRecordButton.Enabled := Caching;
  SetSavePointButton.Enabled := Caching;
  RevertToSavePointButton.Enabled := Caching;
  ViewXMLButton.Enabled := Active;
  SaveToFileButton.Enabled := Active;
  SaveWithParamsButton.Enabled := Active;
  LoadFromFileButton.Enabled := Loadable;
  LoadWithOptionalParams.Enabled := Loadable;
  ShowDeltaButton.Enabled := Caching;
  EnableDisableChangeCache.Enabled := Active;
end;

procedure TForm1.SetSavePointButtonClick(Sender: TObject);
begin
SavedSavePoint := ClientDataSet1.SavePoint;
end;

procedure TForm1.ShowDeltaButtonClick(Sender: TObject);
begin
  with TCacheForm.Create(nil) do
  begin
    ClientDataSet1.Data := Self.ClientDataSet1.Delta;
    ShowModal;
    Release;
  end;
end;

procedure TForm1.RevertToSavePointButtonClick(Sender: TObject);
begin
  if SavedSavePoint = -1 then
    ShowMessage('No save point to revert to')
  else
    try
      ClientDataSet1.SavePoint := SavedSavePoint;
    finally
      SavedSavePoint := -1;
    end;
end;

procedure TForm1.LoadFromFileButtonClick(Sender: TObject);
begin
  ClientDataSet1.LoadFromFile(XMLFileName);
  OpenCloseButton.Caption := 'Close ClientDataSet';
  UpdateButtonStatuses;
end;

procedure TForm1.DataSource1DataChange(Sender: TObject; Field: TField);
begin
  UpdateButtonStatuses;
end;

procedure TForm1.FilterChangedRecordsClick(Sender: TObject);
begin
if FilterChangedRecords.Checked then
  ClientDataSet1.StatusFilter := [usDeleted, usInserted, usModified]
else
  ClientDataSet1.StatusFilter := [];
end;

procedure TForm1.ViewXMLButtonClick(Sender: TObject);
var
  sl: TStringList;
begin
  sl := TStringlist.Create;
  try
    sl.Text := ClientDataSet1.XMLData;
    sl.SaveToFile(Datadir + 'data.xml');
    ShellExecute(Application.Handle, 'open',
      PChar(DataDir + 'data.xml'), nil, nil, SW_SHOW);
  finally
    sl.Free;
  end;
end;

end.
