(* Copyright (c) 2011. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: ClientDataSets" by Cary Jensen

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: ClientDataSets"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/cdsbook
*)
unit mainformu;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DBXDataSnap, DBXCommon, SqlExpr, DB, DBClient, DSConnect, Grids,
  DBGrids, ExtCtrls, DBCtrls, Menus, StdCtrls;

type
  TForm1 = class(TForm)
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    DataSource1: TDataSource;
    ClientDataSet1: TClientDataSet;
    DSProviderConnection1: TDSProviderConnection;
    SQLConnection1: TSQLConnection;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    N1: TMenuItem;
    ApplyUpdates1: TMenuItem;
    N2: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    TableNameComboBox: TComboBox;
    SQLMemo: TMemo;
    OpenDialog1: TOpenDialog;
    Data1: TMenuItem;
    LoadDatatoPass1: TMenuItem;
    procedure Open1Click(Sender: TObject);
    procedure ApplyUpdates1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure DataSource1DataChange(Sender: TObject; Field: TField);
    procedure ClientDataSet1BeforeGetRecords(Sender: TObject;
      var OwnerData: OleVariant);
    procedure LoadDatatoPass1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ClientDataSet1AfterGetRecords(Sender: TObject;
      var OwnerData: OleVariant);
  private
    FTitle: String;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.ApplyUpdates1Click(Sender: TObject);
begin
  ClientDataSet1.ApplyUpdates(0);
  ApplyUpdates1.Enabled := False;
end;

procedure TForm1.ClientDataSet1AfterGetRecords(Sender: TObject;
  var OwnerData: OleVariant);
begin
  if not VarIsEmpty(OwnerData) then
  begin
    ClientDataSet1.ReadOnly := OwnerData;
    if ClientDataSet1.ReadOnly then
      Self.Caption := FTitle + ' - ReadOnly'
    else
      Self.Caption := FTitle
  end;
end;

procedure TForm1.ClientDataSet1BeforeGetRecords(Sender: TObject;
  var OwnerData: OleVariant);
var
  ms: TMemoryStream;
  TheFile: OleVariant;
  TheFilePointer : Pointer;
begin
  if FileExists(OpenDialog1.FileName) then
  begin
    ms := TMemoryStream.Create;
    try
      ms.LoadFromFile(OpenDialog1.FileName);
      TheFile := VarArrayCreate([0, ms.Size - 1], varByte);
      TheFilePointer := VarArrayLock(TheFile);
      try
        ms.ReadBuffer(TheFilePointer^, ms.Size);
      finally
        VarArrayUnlock(TheFile);
      end;
      OwnerData := VarArrayCreate([0,1],VarVariant);
      OwnerData[0] := ExtractFileName(OpenDialog1.FileName);
      OwnerData[1] := TheFile;
    finally
      ms.Free;
    end;
    OpenDialog1.FileName := '';
  end;
end;

procedure TForm1.DataSource1DataChange(Sender: TObject; Field: TField);
begin
  ApplyUpdates1.Enabled := (ClientDataSet1.ChangeCount > 0)
                            and
                            not DBGrid1.ReadOnly;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  ClientDataSet1.Active := False;
  if SQLConnection1.Connected then
    SQLConnection1.Close;
  Close;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FTitle := Self.Caption;
end;

procedure TForm1.LoadDatatoPass1Click(Sender: TObject);
begin
  OpenDialog1.Execute;
end;

procedure TForm1.Open1Click(Sender: TObject);
var
  SQLStmt: String;
begin
  if not ClientDataSet1.Active then
  begin
    if TableNameComboBox.ItemIndex = 0 then
      if SQLMemo.Lines.Text = '' then
        raise Exception.Create('Please supply a SQL statement')
      else
        SQLStmt := SQLMemo.Lines.Text
    else
      SQLStmt := 'SELECT * FROM ' +
        TableNameComboBox.Text;
    //Good to go
    ClientDataSet1.CommandText := SQLStmt;
    ClientDataSet1.Active := True;
    Open1.Caption := 'Close';
    ShowMessage(IntToStr(ClientDataSet1.RecordCount) +
      ' records were found for query ' + SQLStmt);
  end
  else
  begin
    ClientDataSet1.Active := False;
    SQLConnection1.Close;
    Open1.Caption := 'Open';
    ApplyUpdates1.Enabled := False;
  end;
end;

end.
