(* Copyright (c) 2011. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: ClientDataSets" by Cary Jensen

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: ClientDataSets"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/cdsbook
*)
unit IPServerMethodsu;

interface

uses
  SysUtils, Classes, DSServer, Provider, DB, DBTables, Windows, Variants;

type
  TServerMethods1 = class(TDSServerModule)
    Query1: TQuery;
    DataSetProvider1: TDataSetProvider;
    procedure DataSetProvider1BeforeGetRecords(Sender: TObject;
      var OwnerData: OleVariant);
    procedure DataSetProvider1AfterGetRecords(Sender: TObject;
      var OwnerData: OleVariant);
  private
    { Private declarations }
  public
    { Public declarations }
    function EchoString(Value: string): string;
    function ReverseString(Value: string): string;
    {$IFDEF DEBUG}
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    {$ENDIF}
  end;

implementation

{$R *.dfm}

uses StrUtils;

{$IFDEF DEBUG}
constructor TServerMethods1.Create(AOwner: TComponent);
begin
  inherited;
  outputdebugstring(PChar('Starting the server methods object'));
end;

procedure TServerMethods1.DataSetProvider1AfterGetRecords(Sender: TObject;
  var OwnerData: OleVariant);
begin
  OwnerData := not Query1.CanModify;
end;

procedure TServerMethods1.DataSetProvider1BeforeGetRecords(Sender: TObject;
  var OwnerData: OleVariant);
var
  ms: TMemoryStream;
  TheFile: OleVariant;
  TheFilePointer: Pointer;
  TheFileName: String;
begin
  if not VarIsEmpty(OwnerData) then
  begin
    TheFileName := VarToStr(OwnerData[0]);
    TheFile := OwnerData[1];
    ms := TMemoryStream.Create;
    try
      TheFilePointer := VarArrayLock(TheFile);
      try
        ms.Write(TheFilePointer ^, VarArrayHighBound(TheFile, 1));
      finally
        VarArrayUnlock(TheFile);
      end;
      ms.Position := 0;
      ms.SaveToFile('c:\temp\text.txt');
    finally
      ms.Free;
    end;
  end;
end;

destructor TServerMethods1.Destroy;
begin
  outputdebugstring(PChar('stopping the server methods object'));
  inherited;
end;
{$ENDIF}

function TServerMethods1.EchoString(Value: string): string;
begin
  Result := Value;
end;

function TServerMethods1.ReverseString(Value: string): string;
begin
  Result := StrUtils.ReverseString(Value);
end;

end.

