(* Copyright (c) 2011. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: ClientDataSets" by Cary Jensen

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: ClientDataSets"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/cdsbook
*)
unit datamod;

interface

uses
  SysUtils, Classes, DB, DBClient;

type
  TDataModule2 = class(TDataModule)
    VideosCDS: TClientDataSet;
    TalentCDS: TClientDataSet;
    TalentByVideosCDS: TClientDataSet;
    FeaturesByVideosCDS: TClientDataSet;
    procedure DataModuleCreate(Sender: TObject);
    procedure VideosCDSAfterInsert(DataSet: TDataSet);
    procedure TalentCDSAfterInsert(DataSet: TDataSet);
    procedure DataModuleDestroy(Sender: TObject);
    procedure VideosCDSBeforeDelete(DataSet: TDataSet);
  private
    { Private declarations }
  public
    { Public declarations }
    AppDir: String;
  end;

var
  DataModule2: TDataModule2;

implementation

{$R *.dfm}

uses forms, ComObj, ActiveX, Dialogs, Controls;

const
  VideoFile = 'Videos.xml';
  TalentFile = 'Talent.xml';

procedure TDataModule2.DataModuleCreate(Sender: TObject);
begin
  //Identify application location
  Self.AppDir := ExtractFilePath(Application.ExeName);

  //Set data file paths
  TalentCDS.FileName := AppDir + '\' + TalentFile;
  VideosCDS.FileName := AppDir + '\' + VideoFile;

  //Define Talent structure
  begin
    TalentCDS.IndexDefs.Add('IDIndex','ID', [ixPrimary, ixUnique, ixCaseInsensitive]);
    TalentCDS.IndexDefs.Add('NameIndex','Name', [ixCaseInsensitive]);
    TalentCDS.IndexName := 'NameIndex';
    with TStringField.Create(Self) do
    begin
      Name := 'TalentCDSID';
      FieldKind := fkData;
      FieldName := 'ID';

      Size := 42;
      DataSet := TalentCDS;
    end; //ID
    with TStringField.Create(Self) do
    begin
      Name := 'TalentCDSName';
      FieldKind := fkData;
      FieldName := 'Name';
      Size := 42;
      DataSet := TalentCDS;
    end; //Name
    with TMemoField.Create(Self) do
    begin
      Name := 'TalentCDSComments';
      FieldKind := fkData;
      FieldName := 'Comments';
      DataSet := TalentCDS;
    end; //Comments
  end; // if FileExists(TalentCDS.FileName) then ... ELSE
  if FileExists(TalentCDS.FileName) then
    TalentCDS.Open
  else
    TalentCDS.CreateDataSet;

//Define structure of Datasets
  with TStringField.Create(Self) do
  begin
    Name := 'VideosCDSID';
    FieldKind := fkData;
    FieldName := 'ID';
    Size := 42;
    DataSet := VideosCDS;
  end; //ID
  with TStringField.Create(Self) do
  begin
    Name := 'VideosCDSTitle';
    FieldKind := fkData;
    FieldName := 'Title';
    Size := 100;
    DataSet := VideosCDS;
  end; //Title
  with TStringField.Create(Self) do
  begin
    Name := 'VideosCDSFormat';
    FieldKind := fkData;
    FieldName := 'Format';
    Size := 16;
    DataSet := VideosCDS;
  end; //Format
  with TStringField.Create(Self) do
  begin
    Name := 'VideosCDSCategory';
    FieldKind := fkData;
    FieldName := 'Category';
    Size := 16;
    DataSet := VideosCDS;
  end; //Format
  with TStringField.Create(Self) do
  begin
    Name := 'VideosCDSDuration';
    FieldKind := fkData;
    FieldName := 'Duration';
    Size := 16;
    DataSet := VideosCDS;
  end; //Format
  with TStringField.Create(Self) do
  begin
    Name := 'VideosCDSDistributedBy';
    FieldKind := fkData;
    FieldName := 'DistributedBy';
    Size := 72;
    DataSet := VideosCDS;
  end; //Format
  with TMemoField.Create(Self) do
  begin
    Name := 'VideosCDSDescription';
    FieldKind := fkData;
    FieldName := 'Description';
    DataSet := VideosCDS;
  end; //Format

  //Note: For TDataSetFields, FieldKind is fkDataSet by default
  with TDataSetField.Create(Self) do
  begin
    Name := 'VideosCDSTalentByVideo';
    FieldName := 'TalentByVideo';
    DataSet := VideosCDS;
  end; //Format

  with TDataSetField.Create(Self) do
  begin
    Name := 'VideosCDSFeaturesByVideo';
    FieldName := 'FeaturesByVideo';
    DataSet := VideosCDS;
  end; //Format

  with TAggregateField.Create(Self) do
  begin
    Name := 'VideosCDSSTotalTitles';
    FieldKind := fkAggregate;
    FieldName := 'TotalTitles';
    DataSet := VideosCDS;
    Expression := 'Count(ID)';
    GroupingLevel := 0;
    IndexName := 'IDIndex';
    Active := True;
  end; //Format

  //Define the primary index
  VideosCDS.IndexDefs.Add('IDIndex','ID',[ixPrimary, ixUnique, ixCaseInsensitive]);
  VideosCDS.IndexDefs.Add('TitleIndex','Title',[ixCaseInsensitive]);
  VideosCDS.IndexName := 'TitleIndex';


  //Define nested datasets

  //TalentByVideosCDS
  TalentByVideosCDS.DataSetField := TDataSetField(FindComponent('VideosCDSTalentByVideo'));
  with TStringField.Create(Self) do
  begin
    Name := 'TalentByVideosID';
    FieldKind := fkData;
    FieldName := 'TalentID';
    Size := 42;
    DataSet := TalentByVideosCDS;
    Required := True;
  end; //ID
  with TStringField.Create(Self) do
  begin
    Name := 'TalentByVideosName';
    FieldKind := fklookup;
    FieldName := 'Name';
    Size := 50;
    DataSet := TalentByVideosCDS;
    KeyFields := 'TalentID';
    LookupDataSet := TalentCDS;
    LookupKeyFields := 'ID';
    LookupResultField := 'Name';
  end; //ID
  with TMemoField.Create(Self) do
  begin
    Name := 'TalentByVideosComment';
    FieldKind := fkData;
    FieldName := 'Comment';
    DataSet := TalentByVideosCDS;
  end; //ID

  //FeaturesByVideosCDS
  FeaturesByVideosCDS.DataSetField := TDataSetField(FindComponent('VideosCDSFeaturesByVideo'));
  with TStringField.Create(Self) do
  begin
    Name := 'FeaturesByVideosCDSSpecialFeature';
    FieldKind := fkData;
    FieldName := 'SpecialFeature';
    Size := 72;
    DataSet := FeaturesByVideosCDS;
  end; //SpecialFeature
  with TStringField.Create(Self) do
  begin
    Name := 'FeaturesByVideosCDSDescription';
    FieldKind := fkData;
    FieldName := 'Description';
    Size := 72;
    DataSet := FeaturesByVideosCDS;
  end; //Description

  if FileExists(VideosCDS.FileName) then
    VideosCDS.Open
  else
      VideosCDS.CreateDataSet;
  VideosCDS.AggregatesActive := True;

end;

procedure TDataModule2.VideosCDSAfterInsert(DataSet: TDataSet);
var
  GUId: TGuid;
begin
  if CoCreateGuid(GUId) = s_OK then
    VideosCDS.Fields[0].AsString := GuidToString(GUId)
  else
    raise Exception.Create('Cannot create unique key for Video file. Fatal error.');
end;

procedure TDataModule2.TalentCDSAfterInsert(DataSet: TDataSet);
var
  GUId: TGuid;
begin
  if CoCreateGuid(GUId) = 0 then
    TalentCDS.Fields[0].AsString := GuidToString(GUId)
  else
    raise Exception.Create('Cannot create unique key for Talent file. Fatal error.');
end;

procedure TDataModule2.DataModuleDestroy(Sender: TObject);
begin
with VideosCDS do
begin
  if State in [dsEdit, dsInsert] then
    Post;
  MergeChangeLog;
  SaveToFile;
end;
VideosCDS.SaveToFile;
with TalentCDS do
begin
  if State in [dsEdit, dsInsert] then
    Post;
  MergeChangeLog;
  SaveToFile;
end;
end;

procedure TDataModule2.VideosCDSBeforeDelete(DataSet: TDataSet);
var
  i: Integer;
  s: Array of Integer;
  DeleteNested: Boolean;
begin
DeleteNested := False;

//Check for nested datasets
for i := 0 to Pred(VideosCDS.Fields.Count) do
  if DataModule2.VideosCDS.Fields[i].DataType = ftDataSet then
  begin
    SetLength(s, Length(s) + 1);
    s[Pred(Length(s))] := i;
  end;

if Length(s) = 0 then Exit;

//s contains the positions of the NestedDataSets in Fields
for i := 0 to Pred(Length(s)) do
  begin
  if TDataSetField(DataModule2.VideosCDS.Fields[s[i]]).NestedDataSet.RecordCount > 0 then
    begin
    if MessageDlg('At least one detail table contains records. '+
       'Select OK to delete video and all related detail data?', mtConfirmation, mbOKCancel, 0) = mrOK then
       begin
         DeleteNested := True;
         Break;
       end
       else
         //Cancel delete operation without showing an exception
         Abort
    end; // if RecordCount > 0
  end; //for i

if DeleteNested then
  for i := 0 to Pred(Length(s)) do
    while TDataSetField(DataModule2.VideosCDS.Fields[s[i]]).NestedDataSet.RecordCount > 0 do
      TDataSetField(DataModule2.VideosCDS.Fields[s[i]]).NestedDataSet.Delete;
end;

end.
